local flags={"tank","dps","healer","heroic","deadly","important","interruptible","magic","curse","poison","disease","enrage","enrage"}


local EJ_DIFF_10MAN		 		= 1
local EJ_DIFF_25MAN		 		= 2
local EJ_DIFF_10MAN_HEROIC 		= 3
local EJ_DIFF_25MAN_HEROIC 		= 4
local EJ_DIFF_LFRAID	 		= 5

local EJ_SelectInstance, EJ_GetInstanceInfo, EJ_GetEncounterInfo, EJ_GetEncounterInfoByIndex, EJ_InstanceIsRaid, EJ_GetSectionInfo, EJ_SetDifficulty = EJ_SelectInstance, EJ_GetInstanceInfo, EJ_GetEncounterInfo, EJ_GetEncounterInfoByIndex, EJ_InstanceIsRaid, EJ_GetSectionInfo, EJ_SetDifficulty
local tcon = table.concat

local function stringify(s)
	if s==nil then return ""
	elseif type(s)=="boolean" then return s and "true" or "false"
	elseif type(s)~="string" then return ""..s
	else return s end
end

local function escapeA(s)
	s=stringify(s)
	return s:gsub("&","&amp;"):gsub("\"","&quot;"):gsub("\n"," ");
end

local function escapeE(s)
	s=stringify(s)
	return s:gsub("&","&amp;"):gsub("<","&lt;")
end

local function getSectionModes(sectionId)
	EJ_SetDifficulty(EJ_DIFF_25MAN)
	local _,_,_,_,_,_,_,filtered=EJ_GetSectionInfo(sectionId)
	local n25=not filtered
	EJ_SetDifficulty(EJ_DIFF_25MAN_HEROIC)
	_,_,_,_,_,_,_,filtered=EJ_GetSectionInfo(sectionId)
	local h25=not filtered
	EJ_SetDifficulty(EJ_DIFF_10MAN)
	_,_,_,_,_,_,_,filtered=EJ_GetSectionInfo(sectionId)
	local n10=not filtered
	EJ_SetDifficulty(EJ_DIFF_10MAN_HEROIC)
	_,_,_,_,_,_,_,filtered=EJ_GetSectionInfo(sectionId)
	local h10=not filtered
	
	EJ_SetDifficulty(EJ_DIFF_LFRAID)
	_,_,_,_,_,_,_,filtered=EJ_GetSectionInfo(sectionId)
	local lfr=not filtered
	
	local s
	if h25 and h10 and n25 and n10 then s="all"
	elseif h25 and h10 and not n25 and not n10 then s="heroic only"
	elseif n25 and n10 and not h25 and not h10 then s="normal only"
	elseif h25 and n25 and not h10 and not n10 then s="25-man only"
	elseif h10 and n10 and not h25 and not n20 then s="10-man only"
	elseif h25 and not n25 and not h10 and not n10 then s="25-man heroic only"
	elseif h10 and not n10 and not h25 and not n25 then s="10-man heroic only"
	elseif n25 and not h25 and not h10 and not n10 then s="25-man normal only"
	elseif n10 and not h10 and not h25 and not n25 then s="10-man normal only"
	elseif h25 and n25 and h10 and not n10 then s="not in 10-man normal"
	elseif not h25 and not n25 and not h10 and not n10 then s="not used"
	else 
		s=""
		if h25 then s=s.." h25" end
		if n25 then s=s.." n25" end
		if h10 then s=s.." h10" end
		if n10 then s=s.." n10" end
		s=s:sub(2)
	end
	return s,h25,n25,h10,n10,lfr
end

local function processMarkup(s)
	s=s:gsub("|T([^:]*):%d+:%d+:%d+:%d+|t","<icon src=\"%1\"/>")
	s=s:gsub("|CFF(......)([^|]*)|r","<color color=\"#%1\">%2</color>")
	return s
end

local function exportSection(sectionId)
	local title, description, headerType, abilityIcon, displayInfo, siblingId, nextSectionId, 
		filtered, link, startsOpen, flag1, flag2, flag3, flag4 = EJ_GetSectionInfo(sectionId);
		
	local modes=getSectionModes(sectionId)
	EJ_SetDifficulty(EJ_DIFF_25MAN_HEROIC)
		
	local data={}
	data[#data+1]=string.format("<section sectionid=\"%s\"",escapeA(sectionId))
	if abilityIcon and abilityIcon~="" then data[#data+1]=string.format(" abilityIcon=\"%s\"",escapeA(abilityIcon)) end
	
	if flag1 then data[#data+1]=string.format(" flag_%s=\"1\"",flags[flag1+1]) end
	if flag2 then data[#data+1]=string.format(" flag_%s=\"1\"",flags[flag2+1]) end
	if flag3 then data[#data+1]=string.format(" flag_%s=\"1\"",flags[flag3+1]) end
	if flag4 then data[#data+1]=string.format(" flag_%s=\"1\"",flags[flag4+1]) end
	if modes~="all" then
		data[#data+1]=string.format(" modes=\"%s\"",escapeA(modes))
	end
	data[#data+1]=">";
	
	data[#data+1]=string.format("<title>%s</title>",escapeE(title))
	description=processMarkup(escapeE(description))
	data[#data+1]=string.format("<description>%s</description>",description)
	
	if nextSectionId then data[#data+1]=exportSection(nextSectionId) end
	
	data[#data+1]="</section>\n"
	
	if siblingId then data[#data+1]=exportSection(siblingId) end
	
	return tcon(data);
end

local function exportRootSection(sectionId)
	return string.format("<rootsection>%s</rootsection>",exportSection(sectionId))
end

local function exportCreature(creatureIndex)
	local creatureId,creatureName,creatureDescription,displayInfo,icon=EJ_GetCreatureInfo(creatureIndex)
	return string.format("<creature creatureid=\"%s\"><name>%s</name></creature>",escapeA(creatureId),escapeE(creatureName));
end

local function exportEncounter(encounterId)
	local encounterName, encounterDesc, _, rootSectionId = EJ_GetEncounterInfo(encounterId)
	
	local data={}
	data[#data+1]=string.format("<encounter encounterid=\"%s\">",escapeA(encounterId))
	data[#data+1]=string.format("<name>%s</name>",escapeE(encounterName))
	data[#data+1]=string.format("<description>%s</description>",escapeE(encounterDesc))
	
	EJ_SelectEncounter(encounterId)
	
	local i=1
	local _,name=EJ_GetCreatureInfo(i)
	while name and i < 10 do
		data[#data+1]=exportCreature(i)
		i=i+1
		_,name=EJ_GetCreatureInfo(i)
	end
	
	data[#data+1]=exportRootSection(rootSectionId);
	
	data[#data+1]="</encounter>\n"
	return tcon(data);
end

local function exportInstance(instanceId,onlyRaid,onlyEncounter)
	if onlyRaid==nil then onlyRaid=1 end

	EJ_SelectInstance(instanceId)
	
	if onlyRaid == 1 and not EJ_InstanceIsRaid() then return "" end
	if onlyRaid == 2 and EJ_InstanceIsRaid() then return "" end

	local data={}
	data[#data+1]=string.format("<instance instanceid=\"%s\">",escapeA(instanceId))
	
	local instanceName, instanceDescription = EJ_GetInstanceInfo()
	if not onlyEncounter then
		data[#data+1]=string.format("<name>%s</name>",escapeE(instanceName))
		data[#data+1]=string.format("<description>%s</description>",escapeE(instanceDescription));
	end
	
	local i = 1
	local name,_,encounterId = EJ_GetEncounterInfoByIndex(i)

	while name and i < 25 do
		if not onlyEncounter or onlyEncounter==encounterId then
			data[#data+1]=exportEncounter(encounterId)
		end
		i=i+1
		name,_,encounterId=EJ_GetEncounterInfoByIndex(i)
	end
	
	data[#data+1]="</instance>\n";
	return tcon(data);
end

local function header()
	return "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<?xml-stylesheet type=\"text/xsl\" href=\"tohtml.xsl\"?>\n"
end
local function footer()
	return ""
end

local function exportAll(onlyRaids)
	if onlyRaids==nil then onlyRaids=1 end
	
        -- loop through raid instances first
	local i = 1
	local instanceId = EJ_GetInstanceByIndex(i,true)

	local data={}
	data[#data+1]="<ejdata>";
        
	while instanceId do
		data[#data+1]=exportInstance(instanceId,onlyRaids)
		i=i+1
		instanceId = EJ_GetInstanceByIndex(i,true)		
	end

        -- then loop through non-raid instances again
	i = 1
	instanceId = EJ_GetInstanceByIndex(i,false)

	while instanceId do
		data[#data+1]=exportInstance(instanceId,onlyRaids)
		i=i+1
		instanceId = EJ_GetInstanceByIndex(i,false)		
	end

	data[#data+1]="</ejdata>"
	return tcon(data)
end

local function findInstanceByName(name,isRaid)
	if isRaid==nil then
		local id=findInstanceByName(name,true)
		if not id then id=findInstanceByName(name,false) end
		return id
	end

	local i = 1
	local instanceId,instanceName = EJ_GetInstanceByIndex(i,isRaid)
	name=name:lower()

	while instanceId do
		if name==instanceName:lower() then return instanceId end
		i=i+1
		instanceId,instanceName = EJ_GetInstanceByIndex(i,isRaid)		
	end
	return nil
end

local function findEncounterByName(name)
	-- instance must have been selected with EJ_SelectInstance for this to work
	local i = 1
	local encounterName,_,encounterId = EJ_GetEncounterInfoByIndex(i)
	name=name:lower()
	
	while encounterName and i < 25 do
		if name==encounterName:lower() then return encounterId end
		i=i+1
		encounterName,_,encounterId=EJ_GetEncounterInfoByIndex(i)
	end
	return nil
end


local function showText(s)
	local AceGUI=LibStub("AceGUI-3.0")
	local f=AceGUI:Create("Frame")
	f:SetTitle("Encounter journal export")
	f:SetLayout("Fill")
	f:SetCallback('OnClose',function(widget)
		AceGUI:Release(widget)
	end)

	local edit= AceGUI:Create("MultiLineEditBox")
	edit:SetLabel("Copy this to clipboard")
	edit:SetText(s)
	edit.editBox:HighlightText()
	local isAutoFocus=edit.editBox:IsAutoFocus()
	edit.editBox:SetAutoFocus(true)
	edit:SetCallback("OnRelease",function() edit.editBox:SetAutoFocus(isAutoFocus) end)
	f:AddChild(edit)
	
	f:Show()

end

function EJExportAll(onlyRaids)
	local data={}
	data[#data+1]=header()
	data[#data+1]=exportAll(onlyRaids)
	data[#data+1]=footer()
	showText(tcon(data))
end

function EJExportInstance(instanceId)
	if instanceId==nil then
		instanceId=EncounterJournal.instanceID
	elseif type(instanceId)=="string" then
		instanceId=findInstanceByName(instanceId)
		if not instanceId then
			print("Couldn't find instanceId")
			return
		end
	end
	
	data={}
	data[#data+1]=header()
	data[#data+1]="<ejdata>"
	data[#data+1]=exportInstance(instanceId)
	data[#data+1]="</ejdata>"
	data[#data+1]=footer()
	showText(tcon(data))
end

function EJExportEncounter(instanceId,encounterId)
	if instanceId==nil then
		instanceId=EncounterJournal.instanceID
	elseif type(instanceId)=="string" then
		instanceId=findInstanceByName(instanceId)
		if not instanceId then
			print("Couldn't find instanceId")
			return
		end
	end
	if encounterId==nil then
		encounterId=EncounterJournal.encounterID
	elseif type(encounterId)=="string" then
		EJ_SelectInstance(instanceId)
		encounterId=findEncounterByName(encounterId)
		if not encounterId then
			print("Couldn't find encounterId")
			return
		end
	end
	
	data={}
	data[#data+1]=header()
	data[#data+1]="<ejdata>"
	data[#data+1]=exportInstance(instanceId,nil,encounterId)
	data[#data+1]="</ejdata>"
	data[#data+1]=footer()
	showText(tcon(data))
	
end
